#!/usr/bin/Rscript
args <- commandArgs(T)
qcFile <- args[1]
varLim <- as.numeric(args[2])
dpLim <- as.numeric(args[3])
quants <- as.numeric(args[4])
output <- args[5]

read.delim(qcFile, header=T) -> d

vaf <- d$nVar / (d$nRef + d$nVar)

sapply(1:nrow(d), function(x) {
	if (d$del[x] != ".") {
		dels <- matrix(unlist(strsplit(as.character(d$del[x]), c("[:|]"))), byrow=T, ncol=2)
		delRanks <- rank(dels[,2])
		delDp <- as.numeric(dels[rev(delRanks)[1],2])
		if (delDp > varLim) {
			return(T)
		}
	}
	if (d$ins[x] != ".") {
		ins <- matrix(unlist(strsplit(as.character(d$ins[x]), c("[:|]"))), byrow=T, ncol=2)
		insRanks <- rank(ins[,2])
		insDp <- as.numeric(ins[rev(insRanks)[1],2]) 
		if (insDp > varLim) {
			return(T)
		}

	}
	return(F)
}) -> indelSel


#dels <- matrix(unlist(strsplit(d$del, ":|"))
#inss <- strsplit(d$ins, ":|")
#d$vafDel
#d$vafIns 

quantile(vaf, as.numeric(quants), na.rm=T) -> vafCut

sel1 <- vaf > vafCut
sel2 <- d$dp > dpLim
sel3 <- (d$nVar > varLim) | indelSel

sel <- sel1 & sel2 & sel3
write.table(d[sel,], paste(output, "res.sigfilt.tsv", sep="."), row.names=F, col.names=F, quote=F, sep="\t")

dir.create(output)
setwd(output)

png(paste(output, "vafcut", "png", sep="."))
plot(density(log10(vaf), na.rm=T), main=paste("VAF cutoff", vafCut, sep=" "))
abline(v=log10(vafCut), col=2) 
dev.off()

ampls <- unique(d$amp)

pdf(paste(output, "pdf", sep="."), width=14)
sapply(ampls, function(x) {
        sel <- d[,3] == x
        main = x
#       plot(density(as.numeric(as.vector(d[sel,8]))), main=main)
#       hist(as.numeric(as.vector(d[sel,8])), breaks=range(d[sel,8])[2], main=main)

        # Get indels
        dels <- (d[,15] != ".") & sel
        ins <- (d[,14] != ".") & sel

	pNoise <- sum(d$nVar[sel]) / (sum(d$nRef[sel]) + sum(d$nVar[sel]))
	print(pNoise)
	pVals <- ppois(d$nVar[sel], (d$nVar[sel] + d$nRef[sel]) * pNoise, lower.tail=F)
	Qval <- -10*log10(pVals)
	noiseThrs <- qpois(0.01, (d$nVar[sel] + d$nRef[sel]) * pNoise)
        
	plot(d[sel,2], d[sel,4], pch=20, type="s", ylim=c(0, max(d[sel,4])), main=main)
        lines(d[sel,2], d[sel,7], pch=20, type="s", col=2)
        lines(d[sel,2], d[sel,8], pch=20, type="s",col=3)

        ampMax <- max(d[sel,4])

        for (idx in which(dels)) {
                matrix(unlist(strsplit(as.character(as.vector(d[idx,15])), "[:|]")), ncol=2, byrow=T) -> delDat
                nchar(delDat[,1]) -> lngts
                dpths <- as.numeric(delDat[,2])
                rect(d[idx,2] + 1, ampMax - dpths , d[idx,2] + lngts, ampMax, col="#00000040")
        }

        for (idx in which(ins)) {
                matrix(unlist(strsplit(as.character(as.vector(d[idx,14])), "[:|]")), ncol=2, byrow=T) -> insDat
                nchar(insDat[,1]) -> lngts
                dpths <- as.numeric(insDat[,2])
                for (i in 1:ncol(insDat)) {
                polygon(
                        c(d[idx,2], d[idx,2] - (.5 * lngts[i]), d[idx, 2] + (.5 * lngts[i])),
                        c(ampMax, ampMax - dpths[i], ampMax - dpths[i]),
                        , col="#00000040")
                }
        }

        #abline(v=d[dels,2], col=4) 
        #abline(v=d[ins,2], col=5) 

        vaf <- d[sel,8]/(d[sel,7] + d[sel,8])

        plot(density(vaf), main=main)

        hist(vaf, breaks=100, main=main)

        plot(d[sel,2], d[sel,8]/(d[sel,7] + d[sel,8]), pch=20, cex=.5, ylim=c(0,1), main=main)
	points(d[sel,2], noiseThrs / (d[sel,7] + d[sel,8]), col=2, pch=20)
        plot(d[sel,2], log10(d[sel,8]/(d[sel,7] + d[sel,8])), pch=20, cex=.5, ylim=c(-4,0), main=main)
	points(d[sel,2], log10(noiseThrs / (d[sel,7] + d[sel,8])), col=2, pch=20)
        print(x)
#       print(sum(sel))
        })

dev.off()
